/*
 // HSI For Win '95 & NT
 // Uses Win32s 32 bit ASPI dll.
 //
 //     ASPI.c -
 //
 //         contains SCSI/ASPI interface code for the Host SCSI Interface 
 //         library -- for the SEGA Game CARTDEV development environment.
 //     
 //     
 //        Written by the Igneous Group under agreement with Sega Of America.
 //          The Igneous Group
 //          1836 17th Ave, Suite D
 //          Santa Cruz, CA 95062
 //          408 475 8194 voice
 //          408 475 8193 fax
 //         
 //          Geoff Caras   GCaras@igneous.com
 //
 //          Copyright (C) 1995, Sega Of America, Inc.
 //          All Rights Reserved
 //
 //
 //     NOTES
 //            
 //     REFERENCES
 //            
 //     LANGUAGE
 //            ANSI C
 //
 //     LIB          This code comprises the HSI library.
 //            
 //     SEE ALSO The include files (hsi.h) for application info.
 //
 //     These functions are defined here:
 //
 //         EC          HSI_Initialize(BYTE adapter, BYTE id)
 //         static void LookForCartDev(BYTE adapter, BYTE id)
 //
 //     These globals are declared here:
 //
 //         DWORD   ASPIStatus;
 //         BYTE    NumAdapters;
 // 
 //
 //        
 //     MODS
 //     GBC 10 Aug 95   Created from the DOS HSI32 port. Mods for Windows 'Culture'
*/


#include    <windows.h>

//#include    "aspi32ln.h"
#include    "scsidefs.h"
#include    "wnaspi32.h"


//#include    "scsi.h"
#include    "hsi32win.h"
#include    "error.h"

#include    "private.h"

#define  SPACE  ' '


void (*HSI_PostProcP)(void);    // Posting function pointer...

static  void    LookForCartDev(BYTE adapter, BYTE id);

SRB_HAInquiry       HostInqSRB;


DWORD   ASPIStatus;
DWORD   HSI_TimeOut;
HWND    HSI_hwnd;
BYTE    HSIDebug;

EC
HSI_Initialize(BYTE adapter, BYTE id)
{
    
    EC      retcode;

    int     i,
            idx,
            j,
            lun;


    HSI_QID             = 0;
    HSI_QHost           = 0;
    HSI_NumAdapters     = 0;
    CartdevFound        = FALSE;
    HSI_TimeOut         = HSI_DEFAULT_TIMEOUT;

    HSI_CartdevType     = CD_UNKNOWN;         // The type of cartdev found.

    retcode             = err_NOERROR;


    // Initialize the Hosts data structure...
    for(i=0; i < MAX_HOST_ADAPTERS; i++) {
        Hosts[i].HostID        = 0;
        Hosts[i].HostText[0] = '\0';

        for(idx=0; idx<8; idx++) {

            Hosts[i].Targets[idx].DeviceType = 0xff;

            for(j=0; j<32; j++)
                Hosts[i].Targets[idx].InquiryText[j] = SPACE;

            for(j=0; j<8; j++)
                Hosts[i].Targets[idx].Lun[j] = FALSE;        // Lun not allocated.

        } // end for(idx...
    } // end for(i=0...


    ASPIStatus      = GetASPI32SupportInfo();

    HSI_NumAdapters = LOBYTE(LOWORD(ASPIStatus));

    if((HSI_NumAdapters != 0) && (HIBYTE(LOWORD(ASPIStatus)) == SS_COMP)) {
        // ASPI for Win32 is properly initialized!
        ASPI_Inited = 1;
    } else {
        MessageBox(HSI_hwnd,
            "ASPI for Win32 is not initialized!\n\n"
            "Do you want to continue?",
            "HSI32 for Windows",
            MB_ICONSTOP
        );
        retcode = MakeEC(err_LIB, err_NOASPI);
        return(retcode);
    }


    // Do an inquiry on the SCSI Bus devices.

    /* Now we are ready to send ASPI commands (SCSI too!)
     *
     * Lets first figure out how many host adapters there are out there!
    */

	memset(&HostInqSRB, 0, sizeof(HostInqSRB));     // whack the entire structure...
    //memset(&HostInqSRB.HA_ManagerId[0],  SPACE, 16);  // fill String fields with spaces.
    //memset(&HostInqSRB.HA_Identifier[0], SPACE, 16);

    // The data structure has been zero'd out, just fill in the required fields
	HostInqSRB.SRB_Cmd      = SC_HA_INQUIRY;
    HostInqSRB.SRB_Flags    = SRB_DIR_IN;


    LastSRB        = (void *)&HostInqSRB;
    LastSenseP     = NULL;

    /* Now excecute the ASPI command...
    */
	ASPIStatus = SendASPI32Command(&HostInqSRB);

    /* Check the status from the Host Adapter Inquiry
    */
    if(HostInqSRB.SRB_Status != SS_COMP) {
        retcode = MakeEC(err_HOST, err_NOADAPTER);
        HSI_LastError = retcode;
        return(retcode);
    }


    for(HSI_QHost = 0; HSI_QHost < HSI_NumAdapters; HSI_QHost++)  {
            
        HSI_NumAdapters         = HostInqSRB.HA_Count;

        Hosts[HSI_QHost].HostID = HostInqSRB.HA_SCSI_ID;

        idx = 0;

        for(i=0; i<16; i++) 
             Hosts[HSI_QHost].HostText[idx++] = HostInqSRB.HA_ManagerId[i];

        Hosts[HSI_QHost].HostText[idx++]      = SPACE;

        for(i=0; i<16; i++) 
             Hosts[HSI_QHost].HostText[idx++] = HostInqSRB.HA_Identifier[i];

        Hosts[HSI_QHost].HostText[idx++]      = '\0';
         
        /* Now scan the SCSI bus issueing SCSI_INQUIRY commands for each
         * potential ID (0-7) saving the inquiry text as we go...
        */
        HSI_ScanSCSIBus();

    }

    LookForCartDev(adapter, id);

    if(CartdevFound == TRUE) {
        for(lun = 0; lun <= 7; lun++)
            HSI_TestUnitReady((BYTE)lun); // Get rid of the reset condition for each LUN
    }

    return(retcode);
}





static void
LookForCartDev(BYTE adapter, BYTE id)
{
    int     host,
            scsiID;

    char    buf[48];

    char   *p1,
           *p2;

    if((adapter <= 7) && (id <= MAX_HOST_ADAPTERS)) {
        // then there was an id and adapter passed in....
        // verify it...

        host    = adapter;
        scsiID  = id;

        // if that device has not been used go into the scanning code...
        if(Hosts[host].Targets[scsiID].DeviceType != 0xff) {

            // Get a local copy of this ID's inquiry text
            strncpy(buf, Hosts[host].Targets[scsiID].InquiryText, 32);
            buf[32] = '\0';
                                
            // convert it to lower case
            strlwr(buf);
  
            // scan the text in the inquiry string for SEGA OA Saturn CartDev
            p1 = strstr(buf, "sega");
            p2 = strstr(buf, "cartdev");

            if((p1 != NULL) && (p2 != NULL)) {
                // Found the sega text in the inquiry...
                HSI_QHost     = host;
                HSI_QID       = scsiID;
                CartdevFound  = TRUE;
                
                // Look for the type of the cartdev...
                HSI_CartdevType = CD_UNKNOWN;
                
                if(strstr(buf, "mars") != NULL)
                    HSI_CartdevType = CD_MARS;
                else if(strstr(buf, "saturn") != NULL)  {
                    HSI_CartdevType = CD_SATURN;
                }                                    
                // now that the HSI globals have been set we can just return...
                return;
            }
        }
    }


    for(host = 0; host < HSI_NumAdapters; host++) {
        for(scsiID = 0; scsiID < 7; scsiID++) {
            // If the device type is 0xff then this entry has not been used...
            if(Hosts[host].Targets[scsiID].DeviceType == 0xff)
                continue;

            // Get a local copy of this ID's inquiry text
            strncpy(buf, Hosts[host].Targets[scsiID].InquiryText, 32);
            buf[32] = '\0';
                                     
            // convert it to lower case
            strlwr(buf);

            // scan the text in the inquiry string for SEGA OA Saturn CartDev
            p1 = strstr(buf, "sega");
            p2 = strstr(buf, "cartdev");

            if((p1 != NULL) && (p2 != NULL)) {
                // Found the sega text in the inquiry...
                HSI_QHost       = host;
                HSI_QID         = scsiID;
                CartdevFound    = TRUE;
                
                // Look for the type of the cartdev...
                HSI_CartdevType = CD_UNKNOWN;
                
                if(strstr(buf, "mars") != NULL)
                    HSI_CartdevType = CD_MARS;
                else if(strstr(buf, "saturn") != NULL)  {
                    HSI_CartdevType = CD_SATURN;
                }                                    
                
                // now that the HSI globals have been set we can just return...
                return;
            }

        }  // end for(scsiID = 0;

    }  // end for(host = 0;...
}
