/*
 // HSI For Win '95 & NT
 // Uses Win32s 32 bit ASPI dll.
 //
 //    monitor.c -
 //
 //      contains code that interfaces the HSI layer and the target monitor.
 //      It provides the buffer protocol handling and isolates the HSI and SCSI
 //      from the application
 //    
 //
 //    NOTES
 //        
 //    REFERENCES
 //        
 //    LANGUAGE
 //        ANSI C
 //
 //    LIB      This code comprises the HSI library.
 //        
 //    SEE ALSO The include files (hsi.h) for application info.
 //
 //   These functions are defined here:
 //
 //     EC  MON_Initialize();
 //     EC  MON_Stop()
 //     EC  MON_ReadRegisters()
 //     EC  MON_WritRegisters()
 //     EC  MON_ByteRead()
 //     EC  MON_ByteWrite()
 //     EC  MON_WordRead()
 //     EC  MON_WordWrite()
 //     EC  MON_LongRead()
 //     EC  MON_LongWrite()
 //     EC  MON_Go()
 //     EC  MON_Version()
 //     EC  MON_ReadUBC()
 //     EC  MON_WriteUBC()
 //     EC  MON_MakeSafe()
//      EC  GameErrorMap(WORD GameErr)
 //
 //   These globals are declared here:
 // 
 //        
 //    MODS
 //      Written by the Igneous Group under agreement with Sega Of America.
 //        The Igneous Group
 //        1836 17th Avenue, Suite D
 //        Santa Cruz, CA 95062
 //        408 475 8194 voice
 //        408 475 8193 fax
 //        Geoff Caras - GeoffC@igneous.com
 //
 //        Copyright (C) 1995, Sega Of America, Inc.
 //        All Rights Reserved
 //
 //     GBC 15 Aug 95   Ported to the Windows '95 environment.
 //     
*/




#include    <windows.h>

#include    "scsidefs.h"
#include    "wnaspi32.h"

#include    "scsi.h"

#include    "hsi32win.h"
#include    "error.h"

#include    "private.h"

#include    "game.h"
#include    "hsi32mon.h"


#define BufferBusy      MakeEC(err_CARTDEV, err_BufferReady)

BYTE       *MONRam;

CmdHeaderT *MON_CmdP;

ChannelID   CID;

boolean     MonitorMessages;

CartDevBuffIdT  MON_DPROutbuff,
                MON_DPRInbuff;


// Special in-line functions to handle byte swapping...
DWORD  _fastcall
Swap32(DWORD x) {
    _asm xchg   cl, ch
    _asm ror    ecx, 16
    _asm xchg   cl, ch
    _asm mov    x, ecx

    return(x);
}

WORD  _fastcall
Swap16(WORD x) {
    _asm xchg	cl, ch
    _asm mov	x, cx

    return(x);
}




// Initialize and allocate the memory pools that I need.


EC
MON_Initialize(void)
{
    EC  stat;

    MONRam = (BYTE *) GlobalAlloc(GPTR, MON_MEMPOOLSIZE + sizeof(CmdHeaderT) + 16);

    if(MONRam == NULL) {
        return(MakeEC(err_LIB, err_MemoryAllocFail));
    }

    MON_CmdP = (CmdHeaderT *)MONRam;

    MON_MessagesOFF();

    stat = HSI_AllocateChannel(&CID);

    // Only LUN 0 is supported now....
    if(CID != 0)    CID = 0;
    
                         
    MON_DPROutbuff   = 0x0 + 0x10;
    MON_DPRInbuff    = 0x1 + 0x10;

    return(stat);
}

void
MON_ShutDown(void)
{
    GlobalFree(GlobalHandle((LPCVOID)MONRam));
}    



void
MON_MessagesON(void)
{
    MonitorMessages = TRUE;
}

void
MON_MessagesOFF(void)
{
    MonitorMessages = FALSE;
}


EC
MON_Stop(GameuPT uP)
{
    return(IssueMonitorCommand(CMD_Stop, uP, 0, 0, 0));
}

EC
MON_Go(GameuPT uP)
{
    return(IssueMonitorCommand(CMD_Go, uP, 0, 0, 0));
}

EC
MON_MakeSafe(GameuPT uP)
{
    return(IssueMonitorCommand(CMD_MakeSafe, uP, 0, 0, 0));
}

EC
MON_ReadRegisters(GameuPT uP, void *DestRegP)
{
    SH2_RegisterSetT   *RegP,
                       *dP;

    M68K_RegisterSetT  *Reg68P,
                       *d68P;

    EC                  stat;

    DWORD              rsize;
    

    switch(uP) {
        case    uP_SH2_Main:
        case    uP_SH2_Sub:
            rsize = sizeof(SH2_RegisterSetT);
            break;
        case    uP_68000:
            rsize = sizeof(M68K_RegisterSetT);
            break;
        default:
            return(MakeEC(err_MON,  err_BaduP));
    }

    stat = IssueMonitorCommand(CMD_ReadRegisters, uP, 0, 0, rsize);

    if(stat != err_NOERROR)
        return(stat);

    // Swap the data for the destination...

    if((uP == uP_SH2_Main) || (uP == uP_SH2_Sub))  {
        // SH2 normally...
        RegP    = (SH2_RegisterSetT *)((BYTE *)MONRam + sizeof(CmdHeaderT));
        dP      = (SH2_RegisterSetT *)DestRegP;
        dP->R0     = Swap32(RegP->R0);
        dP->R1     = Swap32(RegP->R1);
        dP->R2     = Swap32(RegP->R2);
        dP->R3     = Swap32(RegP->R3);
        dP->R4     = Swap32(RegP->R4);
        dP->R5     = Swap32(RegP->R5);
        dP->R6     = Swap32(RegP->R6);
        dP->R7     = Swap32(RegP->R7);
        dP->R8     = Swap32(RegP->R8);
        dP->R9     = Swap32(RegP->R9);
        dP->R10    = Swap32(RegP->R10);
        dP->R11    = Swap32(RegP->R11);
        dP->R12    = Swap32(RegP->R12);
        dP->R13    = Swap32(RegP->R13);
        dP->R14    = Swap32(RegP->R14);
        dP->R15    = Swap32(RegP->R15);
        dP->PC     = Swap32(RegP->PC);
        dP->PR     = Swap32(RegP->PR);
        dP->GBR    = Swap32(RegP->GBR);
        dP->VBR    = Swap32(RegP->VBR);
        dP->MACL   = Swap32(RegP->MACL);
        dP->MACH   = Swap32(RegP->MACH);
        dP->SR     = Swap32(RegP->SR);
        dP->EXFLAG = Swap16(RegP->EXFLAG);
    } else {
        // 68K registers...
        Reg68P  = (M68K_RegisterSetT *)((BYTE *)MONRam + sizeof(CmdHeaderT));
        d68P    = (M68K_RegisterSetT *)DestRegP;
        d68P->D0    = Swap32(Reg68P->D0);
        d68P->D1    = Swap32(Reg68P->D1);
        d68P->D2    = Swap32(Reg68P->D2);
        d68P->D3    = Swap32(Reg68P->D3);
        d68P->D4    = Swap32(Reg68P->D4);
        d68P->D5    = Swap32(Reg68P->D5);
        d68P->D6    = Swap32(Reg68P->D6);
        d68P->D7    = Swap32(Reg68P->D7);
        d68P->A0    = Swap32(Reg68P->A0);
        d68P->A1    = Swap32(Reg68P->A1);
        d68P->A2    = Swap32(Reg68P->A2);
        d68P->A3    = Swap32(Reg68P->A3);
        d68P->A4    = Swap32(Reg68P->A4);
        d68P->A5    = Swap32(Reg68P->A5);
        d68P->A6    = Swap32(Reg68P->A6);
        d68P->A7    = Swap32(Reg68P->A7);
        d68P->SSP   = Swap32(Reg68P->SSP);
        d68P->USP   = Swap32(Reg68P->USP);

        d68P->SR        = Swap16(Reg68P->SR);
        d68P->EXFLAG    = Swap16(Reg68P->EXFLAG);
        d68P->FUNCCODE  = Swap16(Reg68P->FUNCCODE);
        d68P->ERRORADDR = Swap32(Reg68P->ERRORADDR);
        d68P->INST      = Swap16(Reg68P->INST);
    }

    return(err_NOERROR);
}

EC
MON_WriteRegisters(GameuPT uP, void *RegP)
{
    SH2_RegisterSetT   *DestRegP,
                       *SrcP;

    M68K_RegisterSetT  *DestReg68P,
                       *Src68P;

    EC                  stat;

    DWORD              rsize;
    

    switch(uP) {
        case    uP_SH2_Main:
        case    uP_SH2_Sub:
            rsize = sizeof(SH2_RegisterSetT);
            break;
        case    uP_68000:
            rsize = sizeof(M68K_RegisterSetT);
            break;
        default:
            return(MakeEC(err_MON,  err_BaduP));
    }

    // Swap the data for the destination...

    if((uP == uP_SH2_Main) || (uP == uP_SH2_Sub))  {
        // SH2 normally...
        DestRegP    = (SH2_RegisterSetT *)((BYTE *)MONRam + sizeof(CmdHeaderT));
        SrcP        = (SH2_RegisterSetT *)(RegP);
        DestRegP->R0     = Swap32(SrcP->R0);
        DestRegP->R1     = Swap32(SrcP->R1);
        DestRegP->R2     = Swap32(SrcP->R2);
        DestRegP->R3     = Swap32(SrcP->R3);
        DestRegP->R4     = Swap32(SrcP->R4);
        DestRegP->R5     = Swap32(SrcP->R5);
        DestRegP->R6     = Swap32(SrcP->R6);
        DestRegP->R7     = Swap32(SrcP->R7);
        DestRegP->R8     = Swap32(SrcP->R8);
        DestRegP->R9     = Swap32(SrcP->R9);
        DestRegP->R10    = Swap32(SrcP->R10);
        DestRegP->R11    = Swap32(SrcP->R11);
        DestRegP->R12    = Swap32(SrcP->R12);
        DestRegP->R13    = Swap32(SrcP->R13);
        DestRegP->R14    = Swap32(SrcP->R14);
        DestRegP->R15    = Swap32(SrcP->R15);
        DestRegP->PC     = Swap32(SrcP->PC);
        DestRegP->PR     = Swap32(SrcP->PR);
        DestRegP->GBR    = Swap32(SrcP->GBR);
        DestRegP->VBR    = Swap32(SrcP->VBR);
        DestRegP->MACL   = Swap32(SrcP->MACL);
        DestRegP->MACH   = Swap32(SrcP->MACH);
        DestRegP->SR     = Swap32(SrcP->SR);
        DestRegP->EXFLAG = Swap16(SrcP->EXFLAG);
    } else {
        // 68K registers...
        DestReg68P  = (M68K_RegisterSetT *)((BYTE *)MONRam + sizeof(CmdHeaderT));
        Src68P      = (M68K_RegisterSetT *)(RegP);
        DestReg68P->D0    = Swap32(Src68P->D0);
        DestReg68P->D1    = Swap32(Src68P->D1);
        DestReg68P->D2    = Swap32(Src68P->D2);
        DestReg68P->D3    = Swap32(Src68P->D3);
        DestReg68P->D4    = Swap32(Src68P->D4);
        DestReg68P->D5    = Swap32(Src68P->D5);
        DestReg68P->D6    = Swap32(Src68P->D6);
        DestReg68P->D7    = Swap32(Src68P->D7);
        DestReg68P->A0    = Swap32(Src68P->A0);
        DestReg68P->A1    = Swap32(Src68P->A1);
        DestReg68P->A2    = Swap32(Src68P->A2);
        DestReg68P->A3    = Swap32(Src68P->A3);
        DestReg68P->A4    = Swap32(Src68P->A4);
        DestReg68P->A5    = Swap32(Src68P->A5);
        DestReg68P->A6    = Swap32(Src68P->A6);
        DestReg68P->A7    = Swap32(Src68P->A7);
        DestReg68P->SSP   = Swap32(Src68P->SSP);
        DestReg68P->USP   = Swap32(Src68P->USP);

        DestReg68P->SR        = Swap16(Src68P->SR);
        DestReg68P->EXFLAG    = Swap16(Src68P->EXFLAG);
        DestReg68P->FUNCCODE  = Swap16(Src68P->FUNCCODE);
        DestReg68P->ERRORADDR = Swap32(Src68P->ERRORADDR);
        DestReg68P->INST      = Swap16(Src68P->INST);
    }

    stat = IssueMonitorCommand(CMD_WriteRegisters, uP, rsize, 0, 0);

    return(stat);
}


EC
MON_ByteRead(DWORD From, int len, BYTE *Dest)
{
    EC      stat;

    DWORD  readSize;

    BYTE  *p;

    while(len > 0) { 

        if(len > MON_MEMPOOLSIZE)
            readSize = MON_MEMPOOLSIZE;
        else
            readSize = len;

        stat = IssueMonitorCommand(CMD_ByteRead, uP_SH2_Main, 0, From, readSize);

        if(stat != err_NOERROR)
            return(stat);

        p = ((BYTE *)MONRam + sizeof(CmdHeaderT));

        // Now copy the block over to the destination
        memmove(Dest, p, readSize);

        Dest += readSize;
        From += readSize;
        len  -= readSize;
    }

    return(err_NOERROR);
}

EC
MON_WordRead(DWORD From, int len, BYTE *Dest)
{
    EC      stat;

    DWORD  readSize;

    BYTE  *p;

    while(len > 0) { 

        if(len > MON_MEMPOOLSIZE)
            readSize = MON_MEMPOOLSIZE;
        else
            readSize = len;

        stat = IssueMonitorCommand(CMD_WordRead, uP_SH2_Main, 0, From, readSize);

        if(stat != err_NOERROR)
            return(stat);

        p = ((BYTE *)MONRam + sizeof(CmdHeaderT));

        // Now copy the block over to the destination
        memmove(Dest, p, readSize);

        Dest += readSize;
        From += readSize;
        len  -= readSize;
    }

    return(err_NOERROR);
}

EC
MON_LongRead(DWORD From, int len, BYTE *Dest)
{
    EC      stat;

    DWORD  readSize;

    BYTE  *p;

    while(len > 0) { 

        if(len > MON_MEMPOOLSIZE)
            readSize = MON_MEMPOOLSIZE;
        else
            readSize = len;

        stat = IssueMonitorCommand(CMD_LongRead, uP_SH2_Main, 0, From, readSize);

        if(stat != err_NOERROR)
            return(stat);

        p = ((BYTE *)MONRam + sizeof(CmdHeaderT));

        // Now copy the block over to the destination
        memmove(Dest, p, readSize);

        Dest += readSize;
        From += readSize;
        len  -= readSize;
    }

    return(err_NOERROR);
}


EC
MON_ByteWrite(DWORD Dest, int len, BYTE *From)
{
    EC      stat;

    DWORD  writeSize;

    BYTE   *p;

    while(len > 0) {
        if(len > MON_MEMPOOLSIZE)
            writeSize = MON_MEMPOOLSIZE;
        else
            writeSize = len;

        p = ((BYTE *)MONRam + sizeof(CmdHeaderT));

        memmove(p, From, writeSize);

        stat = IssueMonitorCommand(CMD_ByteWrite, uP_SH2_Main, writeSize, Dest, 0);

        if(stat != err_NOERROR)
            return(stat);

        From += writeSize;
        Dest += writeSize;
        len  -= writeSize;

    }

    return(err_NOERROR);
}


EC
MON_WordWrite(DWORD Dest, int len, BYTE *From)
{
    EC      stat;

    DWORD  writeSize;

    BYTE   *p;

    while(len > 0) {
        if(len > MON_MEMPOOLSIZE)
            writeSize = MON_MEMPOOLSIZE;
        else
            writeSize = len;

        p = ((BYTE *)MONRam + sizeof(CmdHeaderT));

        memmove(p, From, writeSize);

        stat = IssueMonitorCommand(CMD_WordWrite, uP_SH2_Main, writeSize, Dest, 0);

        if(stat != err_NOERROR)
            return(stat);

        From += writeSize;
        Dest += writeSize;
        len  -= writeSize;
    }

    return(err_NOERROR);
}


EC
MON_LongWrite(DWORD Dest, int len, BYTE *From)
{
    EC      stat;

    DWORD  writeSize;

    BYTE   *p;

    while(len > 0) {
        if(len > MON_MEMPOOLSIZE)
            writeSize = MON_MEMPOOLSIZE;
        else
            writeSize = len;

        p = ((BYTE *)MONRam + sizeof(CmdHeaderT));

        memmove(p, From, writeSize);

        stat = IssueMonitorCommand(CMD_LongWrite, uP_SH2_Main, writeSize, Dest, 0);

        if(stat != err_NOERROR)
            return(stat);

        From += writeSize;
        Dest += writeSize;
        len  -= writeSize;

    }

    return(err_NOERROR);
}


EC
MON_GetVersionString(char *DestP)
{
    EC      stat;

    BYTE  *p;

    stat = IssueMonitorCommand(CMD_Version, uP_SH2_Main, 0, 0, 64);

    if(stat != err_NOERROR)
        return(stat);

    p = ((BYTE *)MONRam + sizeof(CmdHeaderT));

    strncat(DestP, p, 64);
    DestP[63] = '\0';

    return(err_NOERROR);
}

EC
MON_ReadUBC(GameuPT uP, UBCT *DestUBCP)
{
    EC                  stat;
    DWORD              rsize;
    UBCT               *ubcP;

    rsize = sizeof(UBCT);

    stat = IssueMonitorCommand(CMD_ReadUBC, uP, 0, 0, rsize);

    if(stat != err_NOERROR)
        return(stat);

    ubcP = (UBCT *)((BYTE *)MONRam + sizeof(CmdHeaderT));

    DestUBCP->BARAH     = Swap16(ubcP->BARAH);
    DestUBCP->BARAL     = Swap16(ubcP->BARAL);
    DestUBCP->BAMRAH    = Swap16(ubcP->BAMRAH);
    DestUBCP->BAMRAL    = Swap16(ubcP->BAMRAL);
    DestUBCP->BBRA      = Swap16(ubcP->BBRA);
    DestUBCP->BARBH     = Swap16(ubcP->BARBH);
    DestUBCP->BARBL     = Swap16(ubcP->BARBL);
    DestUBCP->BAMRBH    = Swap16(ubcP->BAMRBH);
    DestUBCP->BAMRBL    = Swap16(ubcP->BAMRBL);
    DestUBCP->BBRB      = Swap16(ubcP->BBRB);
    DestUBCP->BDRBH     = Swap16(ubcP->BDRBH);
    DestUBCP->BDRBL     = Swap16(ubcP->BDRBL);
    DestUBCP->BDMRBH    = Swap16(ubcP->BDMRBH);
    DestUBCP->BDMRBL    = Swap16(ubcP->BDMRBL);
    DestUBCP->BRCR      = Swap16(ubcP->BRCR);

    return(err_NOERROR);
}

EC
MON_WriteUBC(GameuPT uP, UBCT *SrcUBCP)
{
    EC                  stat;
    DWORD              rsize;
    UBCT               *ubcP;

    rsize = sizeof(UBCT);

    ubcP = (UBCT *)((BYTE *)MONRam + sizeof(CmdHeaderT));

    ubcP->BARAH     = Swap16(SrcUBCP->BARAH);
    ubcP->BARAL     = Swap16(SrcUBCP->BARAL);
    ubcP->BAMRAH    = Swap16(SrcUBCP->BAMRAH);
    ubcP->BAMRAL    = Swap16(SrcUBCP->BAMRAL);
    ubcP->BBRA      = Swap16(SrcUBCP->BBRA);
    ubcP->BARBH     = Swap16(SrcUBCP->BARBH);
    ubcP->BARBL     = Swap16(SrcUBCP->BARBL);
    ubcP->BAMRBH    = Swap16(SrcUBCP->BAMRBH);
    ubcP->BAMRBL    = Swap16(SrcUBCP->BAMRBL);
    ubcP->BBRB      = Swap16(SrcUBCP->BBRB);
    ubcP->BDRBH     = Swap16(SrcUBCP->BDRBH);
    ubcP->BDRBL     = Swap16(SrcUBCP->BDRBL);
    ubcP->BDMRBH    = Swap16(SrcUBCP->BDMRBH);
    ubcP->BDMRBL    = Swap16(SrcUBCP->BDMRBL);
    ubcP->BRCR      = Swap16(SrcUBCP->BRCR);

    stat = IssueMonitorCommand(CMD_ReadUBC, uP, rsize, 0, 0);

    if(stat != err_NOERROR)
        return(stat);

    return(err_NOERROR);
}


EC
IssueMonitorCommand(WORD Command, GameuPT uP, DWORD WriteLen, DWORD DestAddress, DWORD ReadLen)
{
    EC  stat;

    int count;

    // The layer above left a hole for the command header
    MON_CmdP->Command       = Swap16(Command);
    MON_CmdP->Sequence      = 0;
    MON_CmdP->TargetCPU     = Swap16((WORD)uP);
    MON_CmdP->DataLength    = Swap32(WriteLen);
    MON_CmdP->Address       = Swap32(DestAddress);

    // Write the buffer out to DPR output buffer
    stat = HSI_WriteDPR(MON_DPROutbuff, 
                        (CartDevBuffSizeT)(WriteLen + sizeof(CmdHeaderT)), 
                        MONRam, 
                        CID);

    if(stat != err_NOERROR) {
        return(stat);
    }
    

    // Now read back the status from the Saturn... Retry up to count times...
    count = 100;

    do {

        // Now Read the status back from the DPR inbound buffer
        stat = HSI_ReadDPR(MON_DPRInbuff, 
                            (CartDevBuffSizeT)(ReadLen + sizeof(CmdHeaderT)),
                            MONRam, 
                            CID
                          );

    } while((count-- > 0) && (stat == MakeEC(err_CARTDEV, err_BufferReady)));

    if(stat == err_NOERROR)
        stat = GameErrorMap(Swap16(MON_CmdP->ErrorCode));

    return(stat);
}


EC
GameErrorMap(WORD GameErr)
{
    EC  ret;

    switch(GameErr) {
        case merr_NoError:          ret = err_NOERROR;                         break;
        case merr_XferLnMismatch:   ret = MakeEC(err_MON, err_XferLnMismatch); break;
        case merr_InvalidProcessor: ret = MakeEC(err_MON, err_BaduP);          break;
        case merr_InvalidCommand:   ret = MakeEC(err_MON, err_InvalidCommand); break;
        case merr_CPUOffLine:       ret = MakeEC(err_MON, err_CPUOffLine);     break;
        default:                    
            ret = MakeEC(err_MON, err_BadMONError);                    
            break;
    }

    return(ret);
}
